@@ -23,15 +23,13 @@ import ssl
 import sys
 import threading
 import time
-import uuid
 
 import pika
 import pika.exceptions
 
-from cloudify import exceptions
-from cloudify import broker_config
+from cloudify import broker_config, utils
+from cloudify.constants import INSPECT_TIMEOUT
 from cloudify._compat import queue
-from cloudify.constants import EVENTS_EXCHANGE_NAME
 
 # keep compat with both pika 0.11 and pika 1.1: switch the calls based
 # on this flag. We're keeping compat with 0.11 for the py2.6 agent on rhel6.
@@ -551,7 +549,7 @@ class BlockingRequestResponseHandler(TaskConsumer):
     def publish(self, message, correlation_id=None, routing_key='',
                 expiration=None, timeout=None):
         if correlation_id is None:
-            correlation_id = uuid.uuid4().hex
+            correlation_id = utils.uuid4()
         self.make_response_queue(correlation_id)
 
         if expiration is not None:
@@ -618,86 +616,47 @@ def get_client(amqp_host=None,
                connect_timeout=connect_timeout)
 
 
-class CloudifyEventsPublisher(object):
-    SOCKET_TIMEOUT = 5
-    CONNECTION_ATTEMPTS = 3
-    channel_settings = {
-        'auto_delete': False,
-        'durable': True,
-    }
+def is_agent_alive(name,
+                   client,
+                   timeout=INSPECT_TIMEOUT,
+                   connect=True):
+    """
+    Send a `ping` service task to an agent, and validate that a correct
+    response is received
 
-    def __init__(self, amqp_params):
-        self.handlers = {
-            'hook': SendHandler(EVENTS_EXCHANGE_NAME,
-                                exchange_type='topic',
-                                routing_key='events.hooks'),
+    :param name: the agent's amqp exchange name
+    :param client: an AMQPClient for the agent's vhost
+    :param timeout: how long to wait for the response
+    :param connect: whether to connect the client (should be False if it is
+                    already connected)
+    """
+    handler = BlockingRequestResponseHandler(name)
+    client.add_handler(handler)
+    if connect:
+        with client:
+            response = _send_ping_task(name, handler, timeout)
+    else:
+        response = _send_ping_task(name, handler, timeout)
+    return 'time' in response
+
+
+def _send_ping_task(name, handler, timeout=INSPECT_TIMEOUT):
+    task = {
+        'service_task': {
+            'task_name': 'ping',
+            'kwargs': {}
         }
-
-        self._connection = AMQPConnection(
-            handlers=list(self.handlers.values()),
-            amqp_params=amqp_params,
-            name=os.environ.get('AGENT_NAME')
-        )
-        self._is_closed = False
-
-    def connect(self):
-        self._connection.consume_in_thread()
-
-    def publish_message(self, message, message_type):
-        if self._is_closed:
-            raise exceptions.ClosedAMQPClientException(
-                'Publish failed, AMQP client already closed')
-
-        handler = self.handlers.get(message_type)
-
-        if handler:
-            handler.publish(message)
-        else:
-            logger.error('Unknown message type : {0} for message : {1}'.
-                         format(message_type, message))
-
-    def close(self):
-        if self._is_closed:
-            return
-        self._is_closed = True
-        thread = threading.current_thread()
-        if self._connection:
-            logger.debug('Closing amqp client of thread {0}'.format(thread))
-            try:
-                self._connection.close()
-            except Exception as e:
-                logger.debug('Failed to close amqp client of thread {0}, '
-                             'reported error: {1}'.format(thread, repr(e)))
-
-
-def create_events_publisher(amqp_host=None,
-                            amqp_user=None,
-                            amqp_pass=None,
-                            amqp_port=None,
-                            amqp_vhost=None,
-                            ssl_enabled=None,
-                            ssl_cert_path=None,
-                            ssl_cert_data=None):
-    thread = threading.current_thread()
-
-    amqp_params = AMQPParams(
-        amqp_host,
-        amqp_user,
-        amqp_pass,
-        amqp_port,
-        amqp_vhost,
-        ssl_enabled,
-        ssl_cert_path,
-        ssl_cert_data,
-    )
-
+    }
+    # messages expire shortly before we hit the timeout - if they haven't
+    # been handled by then, they won't make the timeout
+    expiration = (timeout * 1000) - 200  # milliseconds
     try:
-        client = CloudifyEventsPublisher(amqp_params)
-        client.connect()
-        logger.debug('AMQP client created for thread {0}'.format(thread))
-    except Exception as e:
-        logger.warning(
-            'Failed to create AMQP client for thread: {0} ({1}: {2})'
-            .format(thread, type(e).__name__, e))
-        raise
-    return client
+        return handler.publish(task, routing_key='service',
+                               timeout=timeout, expiration=expiration)
+    except pika.exceptions.AMQPError as e:
+        logger.warning('Could not send a ping task to {0}: {1}'
+                       .format(name, e))
+        return {}
+    except RuntimeError as e:
+        logger.info('No ping response from {0}: {1}'.format(name, e))
+        return {}
