@@ -295,7 +295,7 @@ class DeploymentContext(EntityContext):
 
     @property
     def resource_tags(self):
-        """Resource tags associated wth this deployment."""
+        """Resource tags associated with this deployment."""
         return self._context.get('deployment_resource_tags')
 
 
@@ -509,6 +509,13 @@ class NodeInstanceContext(EntityContext):
         self._get_node_instance_if_needed()
         return self._node_instance.index
 
+    @property
+    def drift(self):
+        self._get_node_instance_if_needed()
+        drift = self._node_instance.system_properties\
+            .get('configuration_drift')
+        return drift.get('result') if drift.get('ok') else None
+
 
 class RelationshipContext(EntityContext):
     """Holds relationship instance data"""
@@ -980,6 +987,16 @@ class CloudifyContext(CommonContext):
             resource_path=resource_path,
             template_variables=template_variables)
 
+    def download_directory(self,
+                           directory_path,
+                           target_path=None):
+        return self._endpoint.download_directory(
+            blueprint_id=self.blueprint.id,
+            deployment_id=self.deployment.id,
+            resource_path=directory_path,
+            logger=self.logger,
+            target_path=target_path)
+
     def download_resource(self,
                           resource_path,
                           target_path=None):
