@@ -1,4 +1,5 @@
 import os
+import json
 import requests
 
 from cloudify_rest_client.constants import VisibilityState
@@ -25,7 +26,8 @@ class NodeInstance(object):
                  host_id=None,
                  relationships=None,
                  index=None,
-                 scaling_groups=None):
+                 scaling_groups=None,
+                 system_properties=None):
         self.id = node_instance_id
         self._node_id = node_id
         self._runtime_properties = \
@@ -36,6 +38,7 @@ class NodeInstance(object):
         self._relationships = relationships
         self._index = index
         self._scaling_groups = scaling_groups
+        self._system_properties = system_properties
 
     def get(self, key):
         return self._runtime_properties.get(key)
@@ -115,6 +118,10 @@ class NodeInstance(object):
     def scaling_groups(self):
         return {g['id']: g['name'] for g in self._scaling_groups}
 
+    @property
+    def system_properties(self):
+        return self._system_properties
+
 
 def get_rest_client(tenant=None, api_token=None):
     """
@@ -139,7 +146,7 @@ def get_rest_client(tenant=None, api_token=None):
     if execution_token:
         headers[constants.CLOUDIFY_EXECUTION_TOKEN_HEADER] = execution_token
     elif api_token:
-        headers[constants.CLOUDIFY_API_AUTH_TOKEN_HEADER] = api_token
+        token = api_token
     else:
         token = utils.get_rest_token()
 
@@ -293,7 +300,8 @@ def get_resource_from_manager(resource_path,
     )
 
 
-def _resource_paths(blueprint_id, deployment_id, tenant_name, resource_path):
+def _resource_paths(blueprint_id, deployment_id, tenant_name, resource_path,
+                    use_global=True):
     """For the given resource_path, generate all firesever paths to try.
 
     Eg. for path of "foo.txt", generate:
@@ -321,7 +329,8 @@ def _resource_paths(blueprint_id, deployment_id, tenant_name, resource_path):
         resource_path
     ).replace('\\', '/')
 
-    yield resource_path
+    if use_global:
+        yield resource_path
 
 
 def get_resource(blueprint_id, deployment_id, tenant_name, resource_path):
@@ -358,6 +367,30 @@ def get_resource(blueprint_id, deployment_id, tenant_name, resource_path):
                         .format(resource_path))
 
 
+def get_resource_directory_index(
+        blueprint_id, deployment_id, tenant_name, resource_path):
+    tried_paths = []
+    resource_files = set()
+    for path in _resource_paths(
+            blueprint_id, deployment_id, tenant_name, resource_path,
+            use_global=False):
+        try:
+            directory_index = get_resource_from_manager(path)
+            for f in json.loads(directory_index)['files']:
+                resource_files.add(f)
+        except (ValueError, KeyError, NonRecoverableError):
+            tried_paths.append(path)
+        except HttpException as e:
+            if e.code != 404:
+                raise
+
+    if not resource_files:
+        raise HttpException(','.join(tried_paths), 404,
+                            'No valid resource directory listing at found: {0}'
+                            .format(resource_path))
+    return list(resource_files)
+
+
 def get_node_instance(node_instance_id, evaluate_functions=False, client=None):
     """
     Read node instance data from the storage.
@@ -381,7 +414,8 @@ def get_node_instance(node_instance_id, evaluate_functions=False, client=None):
                         host_id=instance.host_id,
                         relationships=instance.relationships,
                         index=instance.index,
-                        scaling_groups=instance.scaling_groups)
+                        scaling_groups=instance.scaling_groups,
+                        system_properties=instance.system_properties)
 
 
 def update_node_instance(node_instance, client=None):
