@@ -19,7 +19,6 @@ import ssl
 import sys
 import time
 import pytz
-import pika
 import shlex
 import random
 import string
@@ -41,8 +40,8 @@ from dsl_parser.constants import PLUGIN_INSTALL_KEY, PLUGIN_NAME_KEY
 from cloudify import constants
 from cloudify.state import workflow_parameters, workflow_ctx, ctx, current_ctx
 from cloudify._compat import StringIO, parse_version
+from cloudify._compat import uuid4  # NOQA - import just to re-export here
 from cloudify.constants import SUPPORTED_ARCHIVE_TYPES
-from cloudify.amqp_client import BlockingRequestResponseHandler
 from cloudify.exceptions import CommandExecutionException, NonRecoverableError
 
 ENV_CFY_EXEC_TEMPDIR = 'CFY_EXEC_TEMP'
@@ -51,7 +50,6 @@ ENV_AGENT_LOG_DIR = 'AGENT_LOG_DIR'
 ENV_AGENT_LOG_MAX_BYTES = 'AGENT_LOG_MAX_BYTES'
 ENV_AGENT_LOG_MAX_HISTORY = 'AGENT_LOG_MAX_HISTORY'
 
-INSPECT_TIMEOUT = 30
 ADMIN_API_TOKEN_PATH = '/opt/mgmtworker/work/admin_token'
 
 
@@ -177,7 +175,7 @@ def format_exception(e):
 
 def get_daemon_name():
     """Name of the currently running agent."""
-    return os.environ['AGENT_NAME']
+    return os.environ.get('AGENT_NAME')
 
 
 def get_manager_name():
@@ -186,7 +184,7 @@ def get_manager_name():
     Available only on mgmtworkers, returns the hostname of the manager
     this is running on.
     """
-    return os.environ[constants.MANAGER_NAME]
+    return os.environ.get(constants.MANAGER_NAME)
 
 
 def get_manager_file_server_scheme():
@@ -595,53 +593,6 @@ class Internal(object):
             ctx._context['tenant'].pop('original_name')
 
 
-def is_agent_alive(name,
-                   client,
-                   timeout=INSPECT_TIMEOUT,
-                   connect=True):
-    """
-    Send a `ping` service task to an agent, and validate that a correct
-    response is received
-
-    :param name: the agent's amqp exchange name
-    :param client: an AMQPClient for the agent's vhost
-    :param timeout: how long to wait for the response
-    :param connect: whether to connect the client (should be False if it is
-                    already connected)
-    """
-    handler = BlockingRequestResponseHandler(name)
-    client.add_handler(handler)
-    if connect:
-        with client:
-            response = _send_ping_task(name, handler, timeout)
-    else:
-        response = _send_ping_task(name, handler, timeout)
-    return 'time' in response
-
-
-def _send_ping_task(name, handler, timeout=INSPECT_TIMEOUT):
-    logger = setup_logger('cloudify.utils.is_agent_alive')
-    task = {
-        'service_task': {
-            'task_name': 'ping',
-            'kwargs': {}
-        }
-    }
-    # messages expire shortly before we hit the timeout - if they haven't
-    # been handled by then, they won't make the timeout
-    expiration = (timeout * 1000) - 200  # milliseconds
-    try:
-        return handler.publish(task, routing_key='service',
-                               timeout=timeout, expiration=expiration)
-    except pika.exceptions.AMQPError as e:
-        logger.warning('Could not send a ping task to {0}: {1}'
-                       .format(name, e))
-        return {}
-    except RuntimeError as e:
-        logger.info('No ping response from {0}: {1}'.format(name, e))
-        return {}
-
-
 def is_management_environment():
     """
     Checks whether we're currently running within a management worker.
@@ -885,6 +836,14 @@ def _find_versioned_plugin_dir(base_dir, version):
         return found
 
 
+def _plugins_base_dir():
+    """The directory where plugins/ and source_plugins/ are stored.
+
+    Default to sys.prefix, which is going to be in the mgmtworker/agent venv.
+    """
+    return os.environ.get('CFY_PLUGINS_ROOT') or sys.prefix
+
+
 def plugin_prefix(name, tenant_name, version=None, deployment_id=None):
     """Virtualenv for the specified plugin.
 
@@ -896,7 +855,7 @@ def plugin_prefix(name, tenant_name, version=None, deployment_id=None):
     :param deployment_id: deployment id, for source plugins
     :return: directory containing the plugin virtualenv, or None
     """
-    managed_plugin_dir = os.path.join(sys.prefix, 'plugins')
+    managed_plugin_dir = os.path.join(_plugins_base_dir(), 'plugins')
     if tenant_name:
         managed_plugin_dir = os.path.join(
             managed_plugin_dir, tenant_name, name)
@@ -905,7 +864,7 @@ def plugin_prefix(name, tenant_name, version=None, deployment_id=None):
 
     prefix = _find_versioned_plugin_dir(managed_plugin_dir, version)
     if prefix is None and deployment_id is not None:
-        source_plugin_dir = os.path.join(sys.prefix, 'source_plugins')
+        source_plugin_dir = os.path.join(_plugins_base_dir(), 'source_plugins')
         if tenant_name:
             source_plugin_dir = os.path.join(
                 source_plugin_dir, tenant_name, deployment_id, name)
@@ -923,7 +882,7 @@ def plugin_prefix(name, tenant_name, version=None, deployment_id=None):
 def target_plugin_prefix(name, tenant_name, version=None, deployment_id=None):
     """Target directory into which the plugin should be installed"""
     parts = [
-        sys.prefix,
+        _plugins_base_dir(),
         'source_plugins' if deployment_id else 'plugins'
     ]
     if tenant_name:
