@@ -17,7 +17,6 @@ import functools
 import time
 import threading
 import types
-import uuid
 
 from cloudify import exceptions, logs
 from cloudify.workflows import api
@@ -38,10 +37,9 @@ from cloudify.constants import (
     TERMINATED_STATES,
 )
 from cloudify.state import workflow_ctx, current_workflow_ctx
-from cloudify.utils import get_func
+from cloudify.utils import get_func, uuid4
 # imported for backwards compat:
-from cloudify.constants import TASK_RESPONSE_SENT  # noqa
-from cloudify.utils import INSPECT_TIMEOUT  # noqa
+from cloudify.constants import TASK_RESPONSE_SENT, INSPECT_TIMEOUT  # noqa
 
 try:
     from functools import lru_cache
@@ -136,7 +134,7 @@ class WorkflowTask(object):
         :param retry_interval: Number of seconds to wait between retries
         :param workflow_context: the CloudifyWorkflowContext instance
         """
-        self.id = task_id or str(uuid.uuid4())
+        self.id = task_id or uuid4()
         self._state = TASK_PENDING
         self.async_result = WorkflowTaskResult(self)
         self.on_success = on_success
@@ -345,6 +343,8 @@ class WorkflowTask(object):
                         time.time() + handler_result.retry_after)
                     handler_result.retried_task = new_task
             else:
+                if self.is_subgraph and handler_result.retried_task:
+                    self.graph.remove_task(handler_result.retried_task)
                 handler_result.action = HandlerResult.HANDLER_FAIL
 
         if self.containing_subgraph:
@@ -430,6 +430,16 @@ class WorkflowTask(object):
         """
         raise NotImplementedError('Implemented by subclasses')
 
+    @property
+    def short_description(self):
+        """Short summary of the task.
+
+        This is a human-readable representation of the task, should be
+        clear and unambiguous enough for the operation to be able to
+        uniquely identify the task.
+        """
+        return self.name
+
     @property
     def is_subgraph(self):
         return False
@@ -534,6 +544,18 @@ class RemoteWorkflowTask(WorkflowTask):
         """The task name"""
         return self.cloudify_context['task_name']
 
+    @property
+    def short_description(self):
+        try:
+            node_id = self.cloudify_context['node_id']
+        except KeyError:
+            node_id = '<unknown node>'
+        try:
+            operation_name = self.cloudify_context['operation']['name']
+        except KeyError:
+            operation_name = '<unknown operation>'
+        return '{0} ({1}:{2})'.format(self.name, node_id, operation_name)
+
     @property
     def cloudify_context(self):
         return self._cloudify_context
@@ -729,6 +751,8 @@ class LocalWorkflowTask(WorkflowTask):
                 self.set_state(TASK_SUCCEEDED, result=result)
                 self.async_result.result = result
             except BaseException as e:
+                if hasattr(e, 'wrapped_exc'):
+                    e = e.wrapped_exc
                 new_task_state = TASK_RESCHEDULED if isinstance(
                     e, exceptions.OperationRetry) else TASK_FAILED
                 self.set_state(new_task_state, exception=e)
