@@ -8,6 +8,7 @@ from cloudify_rest_client import bytes_stream_utils
 from cloudify_rest_client._compat import urlquote, urlparse
 from cloudify_rest_client.constants import VisibilityState
 from cloudify_rest_client.exceptions import CloudifyClientError
+from cloudify_rest_client.executions import Execution
 from cloudify_rest_client.responses import ListResponse
 
 from .labels import Label
@@ -90,6 +91,13 @@ class Blueprint(dict):
         """
         return self.get('upload_execution')
 
+    @property
+    def tenant_name(self):
+        """
+        :return: The name of the tenant associated with this blueprint.
+        """
+        return self.get('tenant_name')
+
 
 class BlueprintsClient(object):
 
@@ -106,7 +114,7 @@ class BlueprintsClient(object):
                              async_upload,
                              labels=None,
                              created_at=None,
-                             owner=None,
+                             created_by=None,
                              state=None,
                              skip_execution=False):
         query_params = {'visibility': visibility, 'async_upload': async_upload,
@@ -129,8 +137,8 @@ class BlueprintsClient(object):
             query_params['labels'] = ','.join(labels_params)
         if created_at:
             query_params['created_at'] = created_at
-        if owner:
-            query_params['owner'] = owner
+        if created_by:
+            query_params['created_by'] = created_by
         if state:
             query_params['state'] = state
 
@@ -159,7 +167,7 @@ class BlueprintsClient(object):
                 async_upload=False,
                 labels=None,
                 created_at=None,
-                owner=None,
+                created_by=None,
                 state=None,
                 skip_execution=False):
         query_params, data = self._prepare_put_request(
@@ -170,7 +178,7 @@ class BlueprintsClient(object):
             async_upload,
             labels,
             created_at,
-            owner,
+            created_by,
             state,
             skip_execution,
         )
@@ -197,11 +205,11 @@ class BlueprintsClient(object):
         )
         uri = '/{self._uri_prefix}/{id}/validate'.format(self=self,
                                                          id=blueprint_id)
-        self.api.put(
+        return self.api.put(
             uri,
             params=query_params,
             data=data,
-            expected_status_code=204
+            expected_status_code=(200, 204)
         )
 
     def _validate_blueprint_size(self, path, tempdir, skip_size_limit):
@@ -236,7 +244,7 @@ class BlueprintsClient(object):
         return tar_path, os.path.basename(path)
 
     def list(self, _include=None, sort=None, is_descending=False,
-             filter_id=None, filter_rules=None, **kwargs):
+             filter_id=None, filter_rules=None, constraints=None, **kwargs):
         """
         Returns a list of currently stored blueprints.
 
@@ -246,10 +254,16 @@ class BlueprintsClient(object):
         :param filter_id: A filter ID to filter the blueprints list by
         :param filter_rules: A list of filter rules to filter the
                blueprints list by
+        :param constraints: A list of DSL constraints for blueprint_id data
+               type.  The purpose is similar to the `filter_rules`, but syntax
+               differs.
         :param kwargs: Optional filter fields. For a list of available fields
                see the REST service's models.BlueprintState.fields
         :return: Blueprints list.
         """
+        if constraints and (filter_id or filter_rules):
+            raise ValueError('provide either DSL constraints or '
+                             'filter_id/filter_rules, not both')
         params = kwargs
         if sort:
             params['_sort'] = '-' + sort if is_descending else sort
@@ -261,6 +275,9 @@ class BlueprintsClient(object):
         if filter_rules:
             response = self.api.post('/searches/blueprints', params=params,
                                      data={'filter_rules': filter_rules})
+        elif constraints:
+            response = self.api.post('/searches/blueprints', params=params,
+                                     data={'constraints': constraints})
         else:
             response = self.api.get('/{self._uri_prefix}'.format(self=self),
                                     params=params)
@@ -278,7 +295,8 @@ class BlueprintsClient(object):
                         async_upload=False,
                         labels=None,
                         created_at=None,
-                        owner=None):
+                        created_by=None,
+                        skip_execution=False):
         """Publishes a blueprint archive to the Cloudify manager.
 
         :param archive_location: Path or Url to the archive file.
@@ -308,7 +326,8 @@ class BlueprintsClient(object):
             async_upload=async_upload,
             labels=labels,
             created_at=created_at,
-            owner=owner)
+            created_by=created_by,
+            skip_execution=skip_execution)
         if not async_upload:
             return self._wrapper_cls(response)
 
@@ -331,7 +350,7 @@ class BlueprintsClient(object):
                async_upload=False,
                labels=None,
                created_at=None,
-               owner=None,
+               created_by=None,
                state=None,
                skip_execution=False):
         """
@@ -368,7 +387,7 @@ class BlueprintsClient(object):
                 async_upload=async_upload,
                 labels=labels,
                 created_at=created_at,
-                owner=owner,
+                created_by=created_by,
                 state=state,
                 skip_execution=skip_execution)
             if not async_upload:
@@ -409,7 +428,7 @@ class BlueprintsClient(object):
                 tar_path, application_file = self._validate_blueprint_size(
                     path, tempdir, skip_size_limit)
 
-            self._validate(
+            response = self._validate(
                 tar_path or path,
                 blueprint_id=entity_id,
                 application_file_name=application_file or blueprint_filename,
@@ -418,6 +437,10 @@ class BlueprintsClient(object):
         finally:
             shutil.rmtree(tempdir)
 
+        if response:
+            # on cloudify earlier than 6.4, response is None (204 no content)
+            return Execution(response)
+
     def get(self, blueprint_id, _include=None):
         """
         Gets a blueprint by its id.
