@@ -1,4 +1,5 @@
 import warnings
+from copy import copy
 
 from cloudify_rest_client.responses import ListResponse
 from cloudify_rest_client.constants import VisibilityState
@@ -17,6 +18,14 @@ class Deployment(dict):
         if 'workflows' in self and self['workflows']:
             # might be None, for example in response for delete deployment
             self['workflows'] = [Workflow(item) for item in self['workflows']]
+        if 'scaling_groups' in self and self['scaling_groups']:
+            self['scaling_groups'] = {
+                name: DeploymentScalingGroup({
+                        'deployment_id': self.id,
+                        'name': name,
+                        'members': spec['members'],
+                        'properties': spec['properties']})
+                for name, spec in self['scaling_groups'].items()}
         if self.get('labels'):
             self['labels'] = [Label(item) for item in self['labels']]
 
@@ -214,6 +223,16 @@ class Deployment(dict):
     def scaling_groups(self):
         return self.get('scaling_groups')
 
+    @property
+    def unavailable_instances(self):
+        """Amount of instances that failed their status check."""
+        return self.get('unavailable_instances')
+
+    @property
+    def drifted_instances(self):
+        """Amount of instances that have configuration drift."""
+        return self.get('drifted_instances')
+
 
 class Workflow(dict):
 
@@ -242,6 +261,16 @@ class Workflow(dict):
         """
         return self['parameters']
 
+    @property
+    def is_available(self):
+        """Is this workflow available for running?"""
+        return self.get('is_available', True)
+
+    @property
+    def availability_rules(self):
+        """Rules defining if this workflow is available"""
+        return self.get('availability_rules')
+
 
 class DeploymentOutputs(dict):
 
@@ -277,6 +306,33 @@ class DeploymentCapabilities(dict):
         return self['capabilities']
 
 
+class DeploymentScalingGroup(dict):
+
+    def __init__(self, scaling_groups):
+        super(DeploymentScalingGroup, self).__init__()
+        self.update(scaling_groups)
+
+    @property
+    def deployment_id(self):
+        """ID of the deployment the capabilities belong to."""
+        return self['deployment_id']
+
+    @property
+    def name(self):
+        """Name of the scaling group."""
+        return self['name']
+
+    @property
+    def members(self):
+        """A list of members of the scaling group (nodes)."""
+        return self['members']
+
+    @property
+    def properties(self):
+        """A dict of scaling group's configuration for instances quantities."""
+        return self['properties']
+
+
 class DeploymentGroup(dict):
     def __init__(self, group):
         super(DeploymentGroup, self).__init__()
@@ -312,9 +368,13 @@ class DeploymentGroupsClient(object):
     def __init__(self, api):
         self.api = api
 
-    def list(self):
+    def list(self, _include=None, **kwargs):
         """List all deployment groups."""
-        response = self.api.get('/deployment-groups')
+        params = kwargs
+        if _include:
+            params['_include'] = ','.join(_include)
+
+        response = self.api.get('/deployment-groups', params=params)
         return ListResponse(
             [DeploymentGroup(item) for item in response['items']],
             response['metadata'])
@@ -328,7 +388,7 @@ class DeploymentGroupsClient(object):
             description=None, blueprint_id=None, default_inputs=None,
             labels=None, filter_id=None, deployment_ids=None,
             new_deployments=None, deployments_from_group=None,
-            creator=None, created_at=None):
+            created_by=None, created_at=None):
         """Create or update the specified deployment group.
 
         Setting group deployments using this method (via either filter_id
@@ -352,7 +412,7 @@ class DeploymentGroupsClient(object):
             keys "id", "inputs", "labels"
         :param deployments_from_group: add all deployments belonging to the
             group given by this id
-        :param creator: Override the creator. Internal use only.
+        :param created_by: Override the creator. Internal use only.
         :param created_at: Override the creation timestamp. Internal use only.
         :return: the created deployment group
         """
@@ -369,8 +429,8 @@ class DeploymentGroupsClient(object):
         }
         if created_at:
             data['created_at'] = created_at
-        if creator:
-            data['creator'] = creator
+        if created_by:
+            data['created_by'] = created_by
         response = self.api.put(
             '/deployment-groups/{0}'.format(group_id), data=data,
         )
@@ -512,6 +572,64 @@ class DeploymentCapabilitiesClient(object):
         response = self.api.get(uri)
         return DeploymentCapabilities(response)
 
+    def list(self, deployment_id, _include=None, constraints=None, **kwargs):
+        """
+        Returns a list of deployment's capabilities matching constraints.
+
+        :param deployment_id: An identifier of a deployment which capabilities
+               are going to be searched.
+        :param _include: List of fields to include in response.
+        :param constraints: A list of DSL constraints for capability_value
+               data type to filter the capabilities by.
+        :param kwargs: Optional filter fields. for a list of available fields
+               see the REST service's models.Deployment.fields
+        :return: Deployments list.
+        """
+        params = kwargs
+        if _include:
+            params['_include'] = ','.join(_include)
+
+        if constraints is None:
+            constraints = dict()
+        constraints['deployment_id'] = deployment_id
+
+        response = self.api.post('/searches/capabilities', params=params,
+                                 data={'constraints': constraints})
+        return ListResponse(
+            items=[DeploymentCapabilities(item) for item in response['items']],
+            metadata=response['metadata']
+        )
+
+
+class DeploymentScalingGroupsClient(object):
+    def __init__(self, api):
+        self.api = api
+
+    def list(self, deployment_id, constraints=None, _include=None, **kwargs):
+        """
+        Returns a list of deployment's scaling groups matching constraints.
+
+        :param deployment_id: An identifier of a deployment which scaling
+               groups are going to be searched.
+        :param constraints: A list of DSL constraints for scaling_group
+               data type to filter the scaling groups by.
+        :param _include: List of fields to include in response.
+        :param kwargs: Optional filter fields. for a list of available fields
+               see the REST service's models.Deployment.fields
+        :return: DeploymentScalingGroup list.
+        """
+        params = copy(kwargs) or {}
+        params['deployment_id'] = deployment_id
+        if _include:
+            params['_include'] = ','.join(_include)
+
+        response = self.api.post('/searches/scaling-groups', params=params,
+                                 data={'constraints': constraints or {}})
+        return ListResponse(
+            items=[DeploymentScalingGroup(item) for item in response['items']],
+            metadata=response['metadata']
+        )
+
 
 class DeploymentsClient(object):
 
@@ -519,9 +637,10 @@ class DeploymentsClient(object):
         self.api = api
         self.outputs = DeploymentOutputsClient(api)
         self.capabilities = DeploymentCapabilitiesClient(api)
+        self.scaling_groups = DeploymentScalingGroupsClient(api)
 
     def list(self, _include=None, sort=None, is_descending=False,
-             filter_id=None, filter_rules=None, **kwargs):
+             filter_id=None, filter_rules=None, constraints=None, **kwargs):
         """
         Returns a list of all deployments.
 
@@ -531,10 +650,16 @@ class DeploymentsClient(object):
         :param filter_id: A filter ID to filter the deployments list by
         :param filter_rules: A list of filter rules to filter the
                deployments list by
+        :param constraints: A list of DSL constraints for deployment_id data
+               type.  The purpose is similar to the `filter_rules`, but syntax
+               differs.
         :param kwargs: Optional filter fields. for a list of available fields
                see the REST service's models.Deployment.fields
         :return: Deployments list.
         """
+        if constraints and (filter_id or filter_rules):
+            raise ValueError('provide either DSL constraints or '
+                             'filter_id/filter_rules, not both')
         params = kwargs
         if sort:
             params['_sort'] = '-' + sort if is_descending else sort
@@ -546,6 +671,9 @@ class DeploymentsClient(object):
         if filter_rules:
             response = self.api.post('/searches/deployments', params=params,
                                      data={'filter_rules': filter_rules})
+        elif constraints:
+            response = self.api.post('/searches/deployments', params=params,
+                                     data={'constraints': constraints})
         else:
             response = self.api.get('/deployments', params=params)
 
@@ -590,6 +718,8 @@ class DeploymentsClient(object):
                labels=None,
                display_name=None,
                async_create=None,
+               created_at=None,
+               created_by=None,
                _workdir_zip=None):
         """
         Creates a new deployment for the provided blueprint id and
@@ -633,6 +763,10 @@ class DeploymentsClient(object):
         data['runtime_only_evaluation'] = runtime_only_evaluation
         uri = '/deployments/{0}'.format(deployment_id)
         params = {}
+        if created_at:
+            data['created_at'] = created_at
+        if created_by:
+            data['created_by'] = created_by
         if async_create is not None:
             # if it's None, we just keep the server's default behaviour
             params['async_create'] = async_create
