@@ -15,7 +15,8 @@
 
 import os
 
-import networkx as nx
+from networkx.algorithms import topological_sort
+from networkx.classes import DiGraph
 
 from cloudify.exceptions import InvalidBlueprintImport
 
@@ -152,13 +153,14 @@ def _dsl_location_to_url(dsl_location, resources_base_path):
 
 def _get_resource_location(resource_name,
                            resources_base_path,
-                           current_resource_context=None):
+                           current_resource_context=None,
+                           dsl_version=None):
     if is_remote_resource(resource_name):
         return resource_name
 
-    if os.path.exists(resource_name):
-        return 'file:{0}'.format(
-            pathname2url(os.path.abspath(resource_name)))
+    for fn in _possible_resource_locations(resource_name, dsl_version):
+        if os.path.exists(fn):
+            return 'file:{0}'.format(pathname2url(os.path.abspath(fn)))
 
     if current_resource_context:
         candidate_url = current_resource_context[
@@ -168,20 +170,32 @@ def _get_resource_location(resource_name,
 
     if resources_base_path:
         full_path = os.path.join(resources_base_path, resource_name)
+        for fn in _possible_resource_locations(full_path, dsl_version):
+            if os.path.exists(fn):
+                return 'file:{0}'.format(pathname2url(os.path.abspath(fn)))
         return 'file:{0}'.format(
             pathname2url(os.path.abspath(full_path)))
 
     return None
 
 
+def _possible_resource_locations(resource_name, dsl_version):
+    if not dsl_version:
+        return [resource_name]
+    filename, ext = os.path.splitext(resource_name)
+    return ['{0}_{1}{2}'.format(filename, dsl_version, ext), resource_name]
+
+
 def _extract_import_parts(import_url,
                           resources_base_path,
-                          current_resource_context=None):
+                          current_resource_context=None,
+                          dsl_version=None):
     """
     :param import_url: a string which is the import path
     :param resources_base_path: In case of a relative file path, this
                                 is the base path.
     :param current_resource_context: Current import statement,
+    :param dsl_version: DSL version used by the root blueprint.
     :return: Will return a breakdown of the URL to
             (namespace, import_url). If the import is not
             namespaced, the returned namespace will be
@@ -196,7 +210,8 @@ def _extract_import_parts(import_url,
 
     return namespace, _get_resource_location(import_url,
                                              resources_base_path,
-                                             current_resource_context)
+                                             current_resource_context,
+                                             dsl_version)
 
 
 def _insert_imported_list(blueprint_holder, blueprints_imported):
@@ -247,6 +262,14 @@ def _combine_imports(parsed_dsl_holder, dsl_location,
     return holder_result
 
 
+def _dsl_version(parsed_dsl_holder):
+    version = parsed_dsl_holder.get_item('tosca_definitions_version')
+    if version:
+        version = version[1].value
+    if version.startswith('cloudify_dsl_'):
+        return version.replace('cloudify_dsl_', '', 1)
+
+
 def _build_ordered_imports(parsed_dsl_holder,
                            dsl_location,
                            resources_base_path,
@@ -347,7 +370,8 @@ def _build_ordered_imports(parsed_dsl_holder,
 
     def build_ordered_imports_recursive(_current_parsed_dsl_holder,
                                         _current_import,
-                                        context_namespace=None):
+                                        context_namespace=None,
+                                        dsl_version=None):
         imports_key_holder, imports_value_holder = _current_parsed_dsl_holder.\
             get_item(constants.IMPORTS)
         if not imports_value_holder:
@@ -356,7 +380,8 @@ def _build_ordered_imports(parsed_dsl_holder,
         for another_import in imports_value_holder.restore():
             namespace, import_url = _extract_import_parts(another_import,
                                                           resources_base_path,
-                                                          _current_import)
+                                                          _current_import,
+                                                          dsl_version)
             validate_namespace(namespace)
             if context_namespace:
                 if namespace:
@@ -392,7 +417,8 @@ def _build_ordered_imports(parsed_dsl_holder,
                     import_context,
                     namespace)
             else:
-                imported_dsl = resolver.fetch_import(import_url)
+                imported_dsl = resolver.fetch_import(import_url,
+                                                     dsl_version=dsl_version)
                 if not is_parsed_resource(imported_dsl):
                     imported_dsl = utils.load_yaml(
                         raw_yaml=imported_dsl,
@@ -401,7 +427,8 @@ def _build_ordered_imports(parsed_dsl_holder,
                                       .format(another_import, import_url),
                         filename=import_url)
                 try:
-                    plugin = resolver.retrieve_plugin(import_url)
+                    plugin = resolver.retrieve_plugin(import_url,
+                                                      dsl_version=dsl_version)
                 except InvalidBlueprintImport:
                     plugin = None
                 if plugin:
@@ -435,14 +462,18 @@ def _build_ordered_imports(parsed_dsl_holder,
                     namespace)
                 build_ordered_imports_recursive(imported_dsl,
                                                 import_url,
-                                                namespace)
+                                                namespace,
+                                                dsl_version)
 
     imports_graph = ImportsGraph()
     blueprint_imports = set()
     namespaces_mapping = {}
 
     imports_graph.add(location(dsl_location), parsed_dsl_holder)
-    build_ordered_imports_recursive(parsed_dsl_holder, dsl_location)
+    build_ordered_imports_recursive(
+        parsed_dsl_holder, dsl_location,
+        dsl_version=_dsl_version(parsed_dsl_holder)
+    )
     sorted_imports_graph = imports_graph.topological_sort()
     return sorted_imports_graph, blueprint_imports, namespaces_mapping
 
@@ -452,7 +483,8 @@ def _validate_version(dsl_version,
                       parsed_imported_dsl_holder):
     version_key_holder, version_value_holder = parsed_imported_dsl_holder\
         .get_item(_version.VERSION)
-    if version_value_holder and version_value_holder.value != dsl_version:
+    if version_value_holder and \
+            not _can_import_version(dsl_version, version_value_holder.value):
         raise exceptions.DSLParsingLogicException(
             28, "An import uses a different "
                 "tosca_definitions_version than the one defined in "
@@ -464,6 +496,12 @@ def _validate_version(dsl_version,
                         version_value_holder.value))
 
 
+def _can_import_version(version_orig, version_imported):
+    """Accept importing a file which uses equal or earlier DSL version."""
+    return _version.SUPPORTED_VERSIONS.index(version_orig) >= \
+        _version.SUPPORTED_VERSIONS.index(version_imported)
+
+
 def _mark_key_value_holder_items(value_holder, field_name, field_value):
     for v in value_holder.value.values():
         setattr(v, field_name, field_value)
@@ -607,6 +645,14 @@ def _extend_node_template(from_dict_holder, to_dict_holder):
                 to_dict_holder.value[key_holder])
 
 
+def _merge_workflows(key_holder, key_name, to_dict_holder, from_dict_holder):
+    source = from_dict_holder.value
+    target = to_dict_holder.value
+    for key in from_dict_holder.value:
+        if key not in target:
+            target[key] = source[key]
+
+
 def _merge_into_dict_or_throw_on_duplicate(from_dict_holder,
                                            to_dict_holder,
                                            key_name,
@@ -637,6 +683,13 @@ def _merge_into_dict_or_throw_on_duplicate(from_dict_holder,
         elif key_name in constants.PLUGIN_DSL_KEYS_READ_FROM_DB:
             # Allow blueprint_labels, labels and resource_tags to be merged
             continue
+        elif key_name == constants.WORKFLOWS:
+            _merge_workflows(
+                key_holder,
+                key_name,
+                to_dict_holder.value[key_holder],
+                value_holder,
+            )
         else:
             raise exceptions.DSLParsingLogicException(
                 4, "Import failed: Could not merge '{0}' due to conflict "
@@ -646,8 +699,8 @@ def _merge_into_dict_or_throw_on_duplicate(from_dict_holder,
 class ImportsGraph(object):
 
     def __init__(self):
-        self._imports_tree = nx.DiGraph()
-        self._imports_graph = nx.DiGraph()
+        self._imports_tree = DiGraph()
+        self._imports_graph = DiGraph()
 
     def add(self, import_url, parsed, cloudify_types=False,
             via_import=None, namespace=None):
@@ -670,7 +723,7 @@ class ImportsGraph(object):
             ({'import': i,
              'parsed': self._imports_tree.node[i]['parsed'],
               'cloudify_types': self._imports_tree.node[i]['cloudify_types']}
-             for i in nx.topological_sort(self._imports_tree))))
+             for i in topological_sort(self._imports_tree))))
 
     def __contains__(self, item):
         return item in self._imports_tree
